@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <ell/ell.h>
 
+#include "src/missing.h"
 #include "src/dbus.h"
 #include "src/netdev.h"
 #include "src/wiphy.h"
@@ -62,8 +63,12 @@ struct wsc {
 	struct {
 		char ssid[33];
 		enum security security;
-		uint8_t psk[32];
+		union {
+			uint8_t psk[32];
+			char passphrase[64];
+		};
 		uint8_t addr[6];
+		bool has_passphrase;
 	} creds[3];
 	uint32_t n_creds;
 	struct l_settings *eap_settings;
@@ -126,14 +131,29 @@ static void wsc_try_credentials(struct wsc *wsc)
 		bss = network_bss_find_by_addr(network, wsc->creds[i].addr);
 
 		if (!bss)
-			bss = network_bss_select(network);
+			bss = network_bss_select(network, true);
 
 		if (!bss)
 			continue;
 
-		if (wsc->creds[i].security == SECURITY_PSK &&
-				!network_set_psk(network, wsc->creds[i].psk))
-			continue;
+		if (wsc->creds[i].security == SECURITY_PSK) {
+			bool ret;
+
+			/*
+			 * Prefer setting passphrase, this will work for both
+			 * WPA2 and WPA3 since the PSK can always be generated
+			 * if needed
+			 */
+			if (wsc->creds[i].has_passphrase)
+				ret = network_set_passphrase(network,
+						wsc->creds[i].passphrase);
+			else
+				ret = network_set_psk(network,
+						wsc->creds[i].psk);
+
+			if (!ret)
+				continue;
+		}
 
 		station_connect_network(wsc->station, network, bss,
 								wsc->pending);
@@ -169,6 +189,7 @@ static void wsc_store_credentials(struct wsc *wsc)
 
 			l_settings_set_value(settings, "Security",
 							"PreSharedKey", hex);
+			explicit_bzero(hex, strlen(hex));
 			l_free(hex);
 		}
 
@@ -204,7 +225,7 @@ static void wsc_disconnect_cb(struct netdev *netdev, bool success,
 }
 
 static void wsc_connect_cb(struct netdev *netdev, enum netdev_result result,
-					void *user_data)
+					void *event_data, void *user_data)
 {
 	struct wsc *wsc = user_data;
 
@@ -252,7 +273,8 @@ static void wsc_credential_obtained(struct wsc *wsc,
 	l_debug("auth_type: %02x, encryption_type: %02x",
 			cred->auth_type, cred->encryption_type);
 
-	l_debug("Key (%u): %.*s", cred->network_key_len,
+	if (getenv("IWD_WSC_DEBUG_KEYS"))
+		l_debug("Key (%u): %.*s", cred->network_key_len,
 				cred->network_key_len, cred->network_key);
 
 	if (wsc->n_creds == L_ARRAY_SIZE(wsc->creds)) {
@@ -309,20 +331,13 @@ static void wsc_credential_obtained(struct wsc *wsc,
 			}
 
 			memcpy(wsc->creds[wsc->n_creds].psk, decoded, 32);
+			explicit_bzero(decoded, 32);
 			l_free(decoded);
 		} else {
-			const char *passphrase =
-				(const char *) cred->network_key;
-			/*
-			 * wscutil should memset cred->network_key to 0 prior
-			 * to copying in the contents of the passphrase
-			 */
-			if (crypto_psk_from_passphrase(passphrase,
-					cred->ssid, cred->ssid_len,
-					wsc->creds[wsc->n_creds].psk) != 0) {
-				l_warn("Ignoring invalid passphrase");
-				return;
-			}
+			strncpy(wsc->creds[wsc->n_creds].passphrase,
+					(const char *) cred->network_key,
+					cred->network_key_len);
+			wsc->creds[wsc->n_creds].has_passphrase = true;
 		}
 
 		break;
@@ -336,6 +351,8 @@ static void wsc_credential_obtained(struct wsc *wsc,
 
 		l_warn("Found duplicate credentials for SSID: %s",
 				wsc->creds[i].ssid);
+		explicit_bzero(&wsc->creds[wsc->n_creds],
+				sizeof(wsc->creds[wsc->n_creds]));
 		return;
 	}
 
@@ -359,7 +376,7 @@ static void wsc_eapol_event(uint32_t event, const void *event_data,
 }
 
 static void wsc_netdev_event(struct netdev *netdev, enum netdev_event event,
-					void *user_data)
+					void *event_data, void *user_data)
 {
 	struct wsc *wsc = user_data;
 
@@ -372,8 +389,8 @@ static void wsc_netdev_event(struct netdev *netdev, enum netdev_event event,
 		break;
 	case NETDEV_EVENT_DISCONNECT_BY_AP:
 		l_debug("Disconnect by AP");
-		wsc_connect_cb(wsc->netdev,
-				NETDEV_RESULT_HANDSHAKE_FAILED, wsc);
+		wsc_connect_cb(wsc->netdev, NETDEV_RESULT_HANDSHAKE_FAILED,
+				event_data, wsc);
 		break;
 	case NETDEV_EVENT_RSSI_THRESHOLD_LOW:
 	case NETDEV_EVENT_RSSI_THRESHOLD_HIGH:
@@ -512,7 +529,8 @@ static void wsc_check_can_connect(struct wsc *wsc, struct scan_bss *target)
 						station_state_watch,
 						wsc, NULL);
 		return;
-	case STATION_STATE_AUTOCONNECT:
+	case STATION_STATE_AUTOCONNECT_QUICK:
+	case STATION_STATE_AUTOCONNECT_FULL:
 	case STATION_STATE_ROAMING:
 		l_warn("wsc_check_can_connect: invalid station state");
 		break;
@@ -528,7 +546,7 @@ static void wsc_cancel_scan(struct wsc *wsc)
 	wsc->wsc_ies = 0;
 
 	if (wsc->scan_id > 0) {
-		scan_cancel(netdev_get_ifindex(wsc->netdev), wsc->scan_id);
+		scan_cancel(netdev_get_wdev_id(wsc->netdev), wsc->scan_id);
 		wsc->scan_id = 0;
 	}
 
@@ -560,8 +578,7 @@ static void pin_timeout(struct l_timeout *timeout, void *user_data)
 					wsc_error_time_expired(wsc->pending));
 }
 
-static bool push_button_scan_results(uint32_t wiphy_id, uint32_t ifindex,
-					int err, struct l_queue *bss_list,
+static bool push_button_scan_results(int err, struct l_queue *bss_list,
 					void *userdata)
 {
 	struct wsc *wsc = userdata;
@@ -659,7 +676,7 @@ static bool push_button_scan_results(uint32_t wiphy_id, uint32_t ifindex,
 		target = bss_2g;
 	else {
 		l_debug("No PBC APs found, running the scan again");
-		wsc->scan_id = scan_active(netdev_get_ifindex(wsc->netdev),
+		wsc->scan_id = scan_active(netdev_get_wdev_id(wsc->netdev),
 						wsc->wsc_ies, wsc->wsc_ies_size,
 						NULL, push_button_scan_results,
 						wsc, NULL);
@@ -718,8 +735,7 @@ static bool authorized_macs_contains(const uint8_t *authorized_macs,
 	return false;
 }
 
-static bool pin_scan_results(uint32_t wiphy_id, uint32_t ifindex, int err,
-				struct l_queue *bss_list, void *userdata)
+static bool pin_scan_results(int err, struct l_queue *bss_list, void *userdata)
 {
 	static const uint8_t wildcard_address[] =
 					{ 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
@@ -806,7 +822,7 @@ static bool pin_scan_results(uint32_t wiphy_id, uint32_t ifindex, int err,
 
 	if (!target) {
 		l_debug("No PIN APs found, running the scan again");
-		wsc->scan_id = scan_active(netdev_get_ifindex(wsc->netdev),
+		wsc->scan_id = scan_active(netdev_get_wdev_id(wsc->netdev),
 						wsc->wsc_ies, wsc->wsc_ies_size,
 						NULL, pin_scan_results,
 						wsc, NULL);
@@ -874,7 +890,7 @@ static bool wsc_initiate_scan(struct wsc *wsc,
 	if (!wsc->wsc_ies)
 		return false;
 
-	wsc->scan_id = scan_active(netdev_get_ifindex(wsc->netdev),
+	wsc->scan_id = scan_active(netdev_get_wdev_id(wsc->netdev),
 					wsc->wsc_ies, wsc->wsc_ies_size,
 					NULL, callback, wsc, NULL);
 	if (!wsc->scan_id) {
@@ -1082,7 +1098,8 @@ static void wsc_netdev_watch(struct netdev *netdev,
 	switch (event) {
 	case NETDEV_WATCH_EVENT_UP:
 	case NETDEV_WATCH_EVENT_NEW:
-		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION)
+		if (netdev_get_iftype(netdev) == NETDEV_IFTYPE_STATION &&
+				netdev_get_is_up(netdev))
 			wsc_add_interface(netdev);
 		break;
 	case NETDEV_WATCH_EVENT_DOWN:
@@ -1094,21 +1111,21 @@ static void wsc_netdev_watch(struct netdev *netdev,
 	}
 }
 
-bool wsc_init(void)
+static int wsc_init(void)
 {
 	l_debug("");
 	netdev_watch = netdev_watch_add(wsc_netdev_watch, NULL, NULL);
 	l_dbus_register_interface(dbus_get_bus(), IWD_WSC_INTERFACE,
 					setup_wsc_interface,
 					wsc_free, false);
-	return true;
+	return 0;
 }
 
-bool wsc_exit()
+static void wsc_exit(void)
 {
 	l_debug("");
 	l_dbus_unregister_interface(dbus_get_bus(), IWD_WSC_INTERFACE);
 	netdev_watch_remove(netdev_watch);
-
-	return true;
 }
+
+IWD_MODULE(wsc, wsc_init, wsc_exit)
